@@ -3,46 +3,146 @@
 /**
  * Rfc822Header.class.php
  *
- * Copyright (c) 2003-2005 The SquirrelMail Project Team
- * Licensed under the GNU GPL. For full terms see the file COPYING.
+ * This file contains functions needed to handle headers in mime messages.
  *
- * This contains functions needed to handle mime messages.
- *
- * $Id: Rfc822Header.class.php,v 1.17.2.20 2005/02/07 00:33:29 kink Exp $
+ * @copyright &copy; 2003-2007 The SquirrelMail Project Team
+ * @license http://opensource.org/licenses/gpl-license.php GNU Public License
+ * @version $Id: Rfc822Header.class.php 13311 2008-10-29 23:05:09Z pdontthink $
+ * @package squirrelmail
+ * @subpackage mime
+ * @since 1.3.2
  */
 
-/*
- * rdc822_header class
+/**
+ * MIME header class
  * input: header_string or array
+ * You must call parseHeader() function after creating object in order to fill object's
+ * parameters.
+ * @todo FIXME: there is no constructor function and class should ignore all input args.
+ * @package squirrelmail
+ * @subpackage mime
+ * @since 1.3.0
  */
 class Rfc822Header {
-    var $date = -1,
-        $subject = '',
-        $from = array(),
-        $sender = '',
-        $reply_to = array(),
-        $mail_followup_to = array(),
-        $to = array(),
-        $cc = array(),
-        $bcc = array(),
-        $in_reply_to = '',
-        $message_id = '',
-        $references = '',
-        $mime = false,
-        $content_type = '',
-        $disposition = '',
-        $xmailer = '',
-        $priority = 3,
-        $dnt = '',
-        $encoding = '',
-        $mlist = array(),
-        $more_headers = array(); /* only needed for constructing headers
-                                    in smtp.php */
+    /**
+     * Date header
+     * @var mixed
+     */
+    var $date = -1;
+    /**
+     * Original date header as fallback for unparsable dates
+     * @var mixed
+     */
+    var $date_unparsed = '';
+    /**
+     * Subject header
+     * @var string
+     */
+    var $subject = '';
+    /**
+     * From header
+     * @var array
+     */
+    var $from = array();
+    /**
+     * @var mixed
+     */
+    var $sender = '';
+    /**
+     * Reply-To header
+     * @var array
+     */
+    var $reply_to = array();
+    /**
+     * Mail-Followup-To header
+     * @var array
+     */
+    var $mail_followup_to = array();
+    /**
+     * To header
+     * @var array
+     */
+    var $to = array();
+    /**
+     * Cc header
+     * @var array
+     */
+    var $cc = array();
+    /**
+     * Bcc header
+     * @var array
+     */
+    var $bcc = array();
+    /**
+     * In-reply-to header
+     * @var string
+     */
+    var $in_reply_to = '';
+    /**
+     * Message-ID header
+     * @var string
+     */
+    var $message_id = '';
+    /**
+     * References header
+     * @var string
+     */
+    var $references = '';
+    /**
+     * @var mixed
+     */
+    var $mime = false;
+    /**
+     * @var mixed
+     */
+    var $content_type = '';
+    /**
+     * @var mixed
+     */
+    var $disposition = '';
+    /**
+     * X-Mailer header
+     * @var string
+     */
+    var $xmailer = '';
+    /**
+     * Priority header
+     * @var integer
+     */
+    var $priority = 3;
+    /**
+     * @var mixed
+     */
+    var $dnt = '';
+    /**
+     * @var mixed
+     */
+    var $encoding = '';
+    /**
+     * @var mixed
+     */
+    var $mlist = array();
+    /**
+     * SpamAssassin 'x-spam-status' header
+     * @var mixed
+     */
+    var $x_spam_status = array();
+    /**
+     * Extra header
+     * only needed for constructing headers in delivery class
+     * @var array
+     */
+    var $more_headers = array();
+
+    /**
+     * @param mixed $hdr string or array with message headers
+     */
     function parseHeader($hdr) {
         if (is_array($hdr)) {
             $hdr = implode('', $hdr);
         }
         /* First we replace \r\n by \n and unfold the header */
+        /* FIXME: unfolding header with multiple spaces "\n( +)" */
         $hdr = trim(str_replace(array("\r\n", "\n\t", "\n "),array("\n", ' ', ' '), $hdr));
 
         /* Now we can make a new header array with */
@@ -58,11 +158,15 @@ class Rfc822Header {
                 }
             }
         }
-        if ($this->content_type == '') {
+        if (!is_object($this->content_type)) {
             $this->parseContentType('text/plain; charset=us-ascii');
         }
     }
 
+    /**
+     * @param string $value
+     * @return string
+     */
     function stripComments($value) {
         $result = '';
         $cnt = strlen($value);
@@ -105,6 +209,11 @@ class Rfc822Header {
         return $result;
     }
 
+    /**
+     * Parse header field according to field type
+     * @param string $field field name
+     * @param string $value field value
+     */
     function parseField($field, $value) {
         $field = strtolower($field);
         switch($field) {
@@ -113,6 +222,7 @@ class Rfc822Header {
                 $d = strtr($value, array('  ' => ' '));
                 $d = explode(' ', $d);
                 $this->date = getTimeStamp($d);
+                $this->date_unparsed = strtr($value,'<>','  ');
                 break;
             case 'subject':
                 $this->subject = $value;
@@ -182,11 +292,11 @@ class Rfc822Header {
                 $this->mlist('post', $value);
                 break;
             case 'list-reply':
-                $value = $this->stripComments($value);            
+                $value = $this->stripComments($value);
                 $this->mlist('reply', $value);
                 break;
             case 'list-subscribe':
-                $value = $this->stripComments($value);            
+                $value = $this->stripComments($value);
                 $this->mlist('subscribe', $value);
                 break;
             case 'list-unsubscribe':
@@ -209,11 +319,18 @@ class Rfc822Header {
                 $value = $this->stripComments($value);
                 $this->mlist('id', $value);
                 break;
+            case 'x-spam-status':
+                $this->x_spam_status = $this->parseSpamStatus($value);
+                break;
             default:
                 break;
         }
     }
 
+    /**
+     * @param string $address
+     * @return array
+     */
     function getAddressTokens($address) {
         $aTokens = array();
         $aAddress = array();
@@ -339,6 +456,14 @@ class Rfc822Header {
         }
         return $aTokens;
     }
+
+    /**
+     * @param array $aStack
+     * @param array $aComment
+     * @param string $sEmail
+     * @param string $sGroup
+     * @return object AddressStructure object
+     */
     function createAddressObject(&$aStack,&$aComment,&$sEmail,$sGroup='') {
         //$aStack=explode(' ',implode('',$aStack));
         if (!$sEmail) {
@@ -348,14 +473,14 @@ class Rfc822Header {
         }
         if (count($aStack)) {
             $sPersonal = trim(implode('',$aStack));
-        } else { 
+        } else {
             $sPersonal = '';
         }
         if (!$sPersonal && count($aComment)) {
             $sComment = trim(implode(' ',$aComment));
             $sPersonal .= $sComment;
         }
-        $oAddr =& new AddressStructure();
+        $oAddr = new AddressStructure();
         if ($sPersonal && substr($sPersonal,0,2) == '=?') {
             $oAddr->personal = encodeHeader($sPersonal);
         } else {
@@ -375,21 +500,8 @@ class Rfc822Header {
         return $oAddr;
     }
 
-    /*
-     * parseAddress: recursive function for parsing address strings and store 
-     *               them in an address stucture object.
-     *               input: $address = string
-     *                      $ar      = boolean (return array instead of only the
-     *                                 first element)
-     *                      $addr_ar = array with parsed addresses // obsolete
-     *                      $group   = string // obsolete
-     *                      $host    = string (default domainname in case of 
-     *                                 addresses without a domainname)
-     *                      $lookup  = callback function (for lookup address
-     *                                 strings which are probably nicks
-     *                                 (without @ ) ) 
-     *               output: array with addressstructure objects or only one
-     *                       address_structure object.
+    /**
+     * recursive function for parsing address strings and storing them in an address stucture object.
      *  personal name: encoded: =?charset?Q|B?string?=
      *                 quoted:  "string"
      *                 normal:  string
@@ -397,8 +509,14 @@ class Rfc822Header {
      *               : mailbox@host
      *  This function is also used for validating addresses returned from compose
      *  That's also the reason that the function became a little bit huge
+     * @param string $address
+     * @param boolean $ar return array instead of only the first element
+     * @param array $addr_ar (obsolete) array with parsed addresses
+     * @param string $group (obsolete)
+     * @param string $host default domainname in case of addresses without a domainname
+     * @param string $lookup (since) callback function for lookup of address strings which are probably nicks (without @)
+     * @return mixed array with AddressStructure objects or only one address_structure object.
      */
-
     function parseAddress($address,$ar=false,$aAddress=array(),$sGroup='',$sHost='',$lookup=false) {
         $aTokens = $this->getAddressTokens($address);
         $sPersonal = $sEmail = $sComment = $sGroup = '';
@@ -410,7 +528,7 @@ class Rfc822Header {
             case '=':
             case '"':
             case ' ':
-                $aStack[] = $sToken; 
+                $aStack[] = $sToken;
                 break;
             case '(':
                 $aComment[] = substr($sToken,1,-1);
@@ -419,9 +537,9 @@ class Rfc822Header {
                 if ($sGroup) {
                     $aAddress[] = $this->createAddressObject($aStack,$aComment,$sEmail,$sGroup);
                     $oAddr = end($aAddress);
-                    if(!$oAddr || ((isset($oAddr)) && !$oAddr->mailbox && !$oAddr->personal)) {
+                    if(!$oAddr || ((isset($oAddr)) && !strlen($oAddr->mailbox) && !$oAddr->personal)) {
                         $sEmail = $sGroup . ':;';
-                    } 
+                    }
                     $aAddress[] = $this->createAddressObject($aStack,$aComment,$sEmail,$sGroup);
                     $sGroup = '';
                     $aStack = $aComment = array();
@@ -430,7 +548,7 @@ class Rfc822Header {
             case ',':
                 $aAddress[] = $this->createAddressObject($aStack,$aComment,$sEmail,$sGroup);
                 break;
-            case ':': 
+            case ':':
                 $sGroup = trim(implode(' ',$aStack));
                 $sGroup = preg_replace('/\s+/',' ',$sGroup);
                 $aStack = array();
@@ -440,7 +558,7 @@ class Rfc822Header {
                break;
             case '>':
                /* skip */
-               break; 
+               break;
             default: $aStack[] = $sToken; break;
             }
         }
@@ -460,8 +578,13 @@ class Rfc822Header {
                          $aAddrBookAddress = $this->parseAddress($aAddr['email'],true);
                      } else {
                          $iPosAt = strpos($aAddr['email'], '@');
-                         $oAddr->mailbox = substr($aAddr['email'], 0, $iPosAt);
-                         $oAddr->host = substr($aAddr['email'], $iPosAt+1);
+                         if ($iPosAt === FALSE) {
+                             $oAddr->mailbox = $aAddr['email'];
+                             $oAddr->host = FALSE;
+                         } else {
+                             $oAddr->mailbox = substr($aAddr['email'], 0, $iPosAt);
+                             $oAddr->host = substr($aAddr['email'], $iPosAt+1);
+                         }
                          if (isset($aAddr['name'])) {
                              $oAddr->personal = $aAddr['name'];
                          } else {
@@ -470,43 +593,51 @@ class Rfc822Header {
                      }
                  }
             }
-            if (!$grouplookup && !$oAddr->mailbox) {
+            if (!$grouplookup && !strlen($oAddr->mailbox)) {
                 $oAddr->mailbox = trim($sEmail);
-                if ($sHost && $oAddr->mailbox) {
+                if ($sHost && strlen($oAddr->mailbox)) {
                     $oAddr->host = $sHost;
                 }
             } else if (!$grouplookup && !$oAddr->host) {
-                if ($sHost && $oAddr->mailbox) {
+                if ($sHost && strlen($oAddr->mailbox)) {
                     $oAddr->host = $sHost;
                 }
-	    }
+            }
           }
-          if (!$aAddrBookAddress && $oAddr->mailbox) {
+          if (!$aAddrBookAddress && strlen($oAddr->mailbox)) {
               $aProcessedAddress[] = $oAddr;
           } else {
-              $aProcessedAddress = array_merge($aProcessedAddress,$aAddrBookAddress); 
+              $aProcessedAddress = array_merge($aProcessedAddress,$aAddrBookAddress);
           }
         }
-        if ($ar) { 
+        if ($ar) {
             return $aProcessedAddress;
         } else {
-            return $aProcessedAddress[0];
+            if (isset($aProcessedAddress[0]))
+                return $aProcessedAddress[0];
+            else
+                return '';
         }
-    } 
+    }
 
     /**
      * Normalise the different Priority headers into a uniform value,
      * namely that of the X-Priority header (1, 3, 5). Supports:
-     * Prioirty, X-Priority, Importance.
+     * Priority, X-Priority, Importance.
      * X-MS-Mail-Priority is not parsed because it always coincides
      * with one of the other headers.
      *
      * NOTE: this is actually a duplicate from the function in
      * functions/imap_messages. I'm not sure if it's ok here to call
      * that function?
+     * @param string $sValue literal priority name
+     * @return integer
      */
-    function parsePriority($value) {
-        $value = strtolower(array_shift(split('/\w/',trim($value))));
+    function parsePriority($sValue) {
+        // don't use function call inside array_shift.
+        $aValue = preg_split('/\s/',trim($sValue));
+        $value = strtolower(array_shift($aValue));
+
         if ( is_numeric($value) ) {
             return $value;
         }
@@ -519,6 +650,9 @@ class Rfc822Header {
         return 3;
     }
 
+    /**
+     * @param string $value content type header
+     */
     function parseContentType($value) {
         $pos = strpos($value, ';');
         $props = '';
@@ -538,39 +672,53 @@ class Rfc822Header {
         }
         $this->content_type = $content_type;
     }
-    
-    /* RFC2184 */
-    function processParameters($aParameters) { 
+
+    /**
+     * RFC2184
+     * @param array $aParameters
+     * @return array
+     */
+    function processParameters($aParameters) {
         $aResults = array();
-	$aCharset = array();
-	// handle multiline parameters
+        $aCharset = array();
+        // handle multiline parameters
         foreach($aParameters as $key => $value) {
-	    if ($iPos = strpos($key,'*')) {
-	        $sKey = substr($key,0,$iPos);
-		if (!isset($aResults[$sKey])) {
-		    $aResults[$sKey] = $value;
-		    if (substr($key,-1) == '*') { // parameter contains language/charset info
-		        $aCharset[] = $sKey;
-		    }
-	        } else {
-		    $aResults[$sKey] .= $value;
-		}
-	    }
+            if ($iPos = strpos($key,'*')) {
+                $sKey = substr($key,0,$iPos);
+                if (!isset($aResults[$sKey])) {
+                    $aResults[$sKey] = $value;
+                    if (substr($key,-1) == '*') { // parameter contains language/charset info
+                        $aCharset[] = $sKey;
+                    }
+                } else {
+                    $aResults[$sKey] .= $value;
+                }
+            } else {
+                $aResults[$key] = $value;
+            }
         }
-	foreach ($aCharset as $key) {
-	    $value = $aResults[$key];
-	    // extract the charset & language
-	    $charset = substr($value,0,strpos($value,"'"));
-	    $value = substr($value,strlen($charset)+1);
-	    $language = substr($value,0,strpos($value,"'"));
-	    $value = substr($value,strlen($charset)+1);
-	    // FIX ME What's the status of charset decode with language information ????
-	    $value = charset_decode($charset,$value);
-	    $aResults[$key] = $value;
-	}
-	return $aResults;    
+        foreach ($aCharset as $key) {
+            $value = $aResults[$key];
+            // extract the charset & language
+            $charset = substr($value,0,strpos($value,"'"));
+            $value = substr($value,strlen($charset)+1);
+            $language = substr($value,0,strpos($value,"'"));
+            $value = substr($value,strlen($charset)+1);
+            /* FIXME: What's the status of charset decode with language information ????
+             * Maybe language information contains only ascii text and charset_decode() 
+             * only runs htmlspecialchars() on it. If it contains 8bit information, you 
+             * get html encoded text in charset used by selected translation.
+             */
+            $value = charset_decode($charset,$value);
+            $aResults[$key] = $value;
+        }
+        return $aResults;
     }
 
+    /**
+     * @param string $value
+     * @return array
+     */
     function parseProperties($value) {
         $propArray = explode(';', $value);
         $propResultArray = array();
@@ -589,6 +737,10 @@ class Rfc822Header {
         return $this->processParameters($propResultArray);
     }
 
+    /**
+     * Fills disposition object in rfc822Header object
+     * @param string $value
+     */
     function parseDisposition($value) {
         $pos = strpos($value, ';');
         $props = '';
@@ -604,6 +756,11 @@ class Rfc822Header {
         $this->disposition = $disp;
     }
 
+    /**
+     * Fills mlist array keys in rfc822Header object 
+     * @param string $field
+     * @param string $value
+     */
     function mlist($field, $value) {
         $res_a = array();
         $value_a = explode(',', $value);
@@ -621,11 +778,61 @@ class Rfc822Header {
         $this->mlist[$field] = $res_a;
     }
 
-    /*
-     * function to get the addres strings out of the header.
-     * Arguments: string or array of strings !
+    /**
+     * Parses the X-Spam-Status header
+     * @param string $value
+     */
+    function parseSpamStatus($value) {
+        // Header value looks like this:
+        // No, score=1.5 required=5.0 tests=MSGID_FROM_MTA_ID,NO_REAL_NAME,UPPERCASE_25_50 autolearn=disabled version=3.1.0-gr0
+
+        $spam_status = array();
+
+        if (preg_match ('/^(No|Yes),\s+score=(-?\d+\.\d+)\s+required=(-?\d+\.\d+)\s+tests=(.*?)\s+autolearn=(.*?)\s+version=(.+?)$/', $value, $matches)) {
+            // full header
+            $spam_status['bad_format'] = 0;
+            $spam_status['value'] = $matches[0];
+            // is_spam
+            if (isset($matches[1])
+                && strtolower($matches[1]) == 'yes') {
+                $spam_status['is_spam'] = true;
+            } else {
+                $spam_status['is_spam'] = false;
+            }
+
+            // score
+            $spam_status['score'] = $matches[2];
+
+            // required
+            $spam_status['required'] = $matches[3];
+
+            // tests
+            $tests = array();
+            $tests = explode(',', $matches[4]);
+            foreach ($tests as $test) {
+                $spam_status['tests'][] = trim($test);
+            }
+
+            // autolearn
+            $spam_status['autolearn'] = $matches[5];
+
+            // version
+            $spam_status['version'] = $matches[6];
+        } else {
+            $spam_status['bad_format'] = 1;
+            $spam_status['value'] = $value;
+        }
+        return $spam_status;
+    }
+
+    /**
+     * function to get the address strings out of the header.
      * example1: header->getAddr_s('to').
      * example2: header->getAddr_s(array('to', 'cc', 'bcc'))
+     * @param mixed $arr string or array of strings
+     * @param string $separator
+     * @param boolean $encoded (since 1.4.0) return encoded or plain text addresses
+     * @return string
      */
     function getAddr_s($arr, $separator = ',',$encoded=false) {
         $s = '';
@@ -663,6 +870,13 @@ class Rfc822Header {
         return $s;
     }
 
+    /**
+     * function to get the array of addresses out of the header.
+     * @param mixed $arg string or array of strings
+     * @param array $excl_arr array of excluded email addresses
+     * @param array $arr array of added email addresses
+     * @return array
+     */
     function getAddr_a($arg, $excl_arr = array(), $arr = array()) {
         if (is_array($arg)) {
             foreach($arg as $argument) {
@@ -697,7 +911,14 @@ class Rfc822Header {
         }
         return $arr;
     }
-    
+
+    /**
+//FIXME: This needs some documentation (inside the function too)!  Don't code w/out comments!
+     * @param mixed $address array or string
+     * @param boolean $recurs
+     * @return mixed array, boolean
+     * @since 1.3.2
+     */
     function findAddress($address, $recurs = false) {
         $result = false;
         if (is_array($address)) {
@@ -712,17 +933,18 @@ class Rfc822Header {
                         $result = $i;
                     }
                 }
-                ++$i;        
+                ++$i;
             }
         } else {
             if (!is_array($this->cc)) $this->cc = array();
+            if (!is_array($this->to)) $this->to = array();
             $srch_addr = $this->parseAddress($address);
             $results = array();
             foreach ($this->to as $to) {
-                if ($to->host == $srch_addr->host) {
-                    if ($to->mailbox == $srch_addr->mailbox) {
+                if (strtolower($to->host) == strtolower($srch_addr->host)) {
+                    if (strtolower($to->mailbox) == strtolower($srch_addr->mailbox)) {
                         $results[] = $srch_addr;
-                        if ($to->personal == $srch_addr->personal) {
+                        if (strtolower($to->personal) == strtolower($srch_addr->personal)) {
                             if ($recurs) {
                                 return array($results, true);
                             } else {
@@ -732,11 +954,11 @@ class Rfc822Header {
                     }
                 }
             }
-             foreach ($this->cc as $cc) {
-                if ($cc->host == $srch_addr->host) {
-                    if ($cc->mailbox == $srch_addr->mailbox) {
+            foreach ($this->cc as $cc) {
+                if (strtolower($cc->host) == strtolower($srch_addr->host)) {
+                    if (strtolower($cc->mailbox) == strtolower($srch_addr->mailbox)) {
                         $results[] = $srch_addr;
-                        if ($cc->personal == $srch_addr->personal) {
+                        if (strtolower($cc->personal) == strtolower($srch_addr->personal)) {
                             if ($recurs) {
                                 return array($results, true);
                             } else {
@@ -752,12 +974,18 @@ class Rfc822Header {
                 return true;
             } else {
                 return false;
-            }        
+            }
         }
         //exit;
         return $result;
     }
 
+    /**
+     * @param string $type0 media type
+     * @param string $type1 media subtype
+     * @return array media properties
+     * @todo check use of media type arguments
+     */
     function getContentType($type0, $type1) {
         $type0 = $this->content_type->type0;
         $type1 = $this->content_type->type1;
@@ -765,4 +993,3 @@ class Rfc822Header {
     }
 }
 
-?>
