@@ -1,11 +1,11 @@
 <?php
+
 /**
  * abook_database.php
  *
- * Copyright (c) 1999-2005 The SquirrelMail Project Team
- * Licensed under the GNU GPL. For full terms see the file COPYING.
- *
- * @version $Id: abook_database.php,v 1.15.2.11 2005/06/04 17:07:28 jervfors Exp $
+ * @copyright &copy; 1999-2007 The SquirrelMail Project Team
+ * @license http://opensource.org/licenses/gpl-license.php GNU Public License
+ * @version $Id: abook_database.php 13188 2008-06-20 07:58:39Z pdontthink $
  * @package squirrelmail
  * @subpackage addressbook
  */
@@ -161,6 +161,13 @@ class abook_database extends addressbook_backend {
         }
 
         $this->dbh = $dbh;
+
+        /**
+         * field names are lowercased.
+         * We use unquoted identifiers and they use upper case in Oracle
+         */
+        $this->dbh->setOption('portability', DB_PORTABILITY_LOWERCASE);
+
         return true;
     }
 
@@ -172,6 +179,34 @@ class abook_database extends addressbook_backend {
         $this->dbh = false;
     }
 
+    /**
+     * Determine internal database field name given one of
+     * the SquirrelMail SM_ABOOK_FIELD_* constants
+     *
+     * @param integer $field The SM_ABOOK_FIELD_* contant to look up
+     *
+     * @return string The desired field name, or the string "ERROR"
+     *                if the $field is not understood (the caller
+     *                is responsible for handing errors)
+     *
+     */
+    function get_field_name($field) {
+        switch ($field) {
+            case SM_ABOOK_FIELD_NICKNAME:
+                return 'nickname';
+            case SM_ABOOK_FIELD_FIRSTNAME:
+                return 'firstname';
+            case SM_ABOOK_FIELD_LASTNAME:
+                return 'lastname';
+            case SM_ABOOK_FIELD_EMAIL:
+                return 'email';
+            case SM_ABOOK_FIELD_LABEL:
+                return 'label';
+            default:
+                return 'ERROR';
+        }
+    }
+
     /* ========================== Public ======================== */
 
     /**
@@ -179,7 +214,7 @@ class abook_database extends addressbook_backend {
      * @param string $expr search expression
      * @return array search results
      */
-    function &search($expr) {
+    function search($expr) {
         $ret = array();
         if(!$this->open()) {
             return false;
@@ -190,15 +225,30 @@ class abook_database extends addressbook_backend {
             return;
         }
 
-        /* Make regexp from glob'ed expression  */
+        // don't allow wide search when listing is disabled.
+        if ($expr=='*' && ! $this->listing) {
+            return array();
+        }
+
+        /* lowercase expression in order to make it case insensitive */
+        $expr = strtolower($expr);
+
+        /* escape SQL wildcards */
+        $expr = str_replace('_', '\\_', $expr);
+        $expr = str_replace('%', '\\%', $expr);
+
+        /* Convert wildcards to SQL syntax  */
         $expr = str_replace('?', '_', $expr);
         $expr = str_replace('*', '%', $expr);
         $expr = $this->dbh->quoteString($expr);
         $expr = "%$expr%";
 
+        /* create escape expression */
+        $escape = 'ESCAPE \'' . $this->dbh->quoteString('\\') . '\'';
+
         $query = sprintf("SELECT * FROM %s WHERE owner='%s' AND " .
-                         "(firstname LIKE '%s' OR lastname LIKE '%s')",
-                         $this->table, $this->owner, $expr, $expr);
+                         "(LOWER(firstname) LIKE '%s' %s OR LOWER(lastname) LIKE '%s' %s)",
+                         $this->table, $this->owner, $expr, $escape, $expr, $escape);
         $res = $this->dbh->query($query);
 
         if (DB::isError($res)) {
@@ -220,23 +270,35 @@ class abook_database extends addressbook_backend {
     }
 
     /**
-     * Lookup alias
-     * @param string $alias alias
+     * Lookup by the indicated field
+     *
+     * @param string  $value Value to look up
+     * @param integer $field The field to look in, should be one
+     *                       of the SM_ABOOK_FIELD_* constants
+     *                       defined in functions/constants.php
+     *                       (OPTIONAL; defaults to nickname field)
+     *                       NOTE: uniqueness is only guaranteed
+     *                       when the nickname field is used here;
+     *                       otherwise, the first matching address
+     *                       is returned.
+     *
      * @return array search results
+     *
      */
-    function &lookup($alias) {
-        if (empty($alias)) {
+    function lookup($value, $field=SM_ABOOK_FIELD_NICKNAME) {
+        if (empty($value)) {
             return array();
         }
 
-        $alias = strtolower($alias);
+        $value = strtolower($value);
 
         if (!$this->open()) {
             return false;
         }
 
-        $query = sprintf("SELECT * FROM %s WHERE owner='%s' AND LOWER(nickname)='%s'",
-                         $this->table, $this->owner, $this->dbh->quoteString($alias));
+        $query = sprintf("SELECT * FROM %s WHERE owner = '%s' AND LOWER(%s) = '%s'",
+                         $this->table, $this->owner, $this->get_field_name($field), 
+                         $this->dbh->quoteString($value));
 
         $res = $this->dbh->query($query);
 
@@ -262,7 +324,7 @@ class abook_database extends addressbook_backend {
      * List all addresses
      * @return array search results
      */
-    function &list_addr() {
+    function list_addr() {
         $ret = array();
         if (!$this->open()) {
             return false;
@@ -303,7 +365,7 @@ class abook_database extends addressbook_backend {
      */
     function add($userdata) {
         if (!$this->writeable) {
-            return $this->set_error(_("Addressbook is read-only"));
+            return $this->set_error(_("Address book is read-only"));
         }
 
         if (!$this->open()) {
@@ -313,8 +375,7 @@ class abook_database extends addressbook_backend {
         /* See if user exist already */
         $ret = $this->lookup($userdata['nickname']);
         if (!empty($ret)) {
-            return $this->set_error(sprintf(_("User %s already exists"),
-                        '&quot;' . $ret['nickname'] . '&quot;'));
+            return $this->set_error(sprintf(_("User \"%s\" already exists"), $ret['nickname']));
         }
 
         /* Create query */
@@ -324,19 +385,20 @@ class abook_database extends addressbook_backend {
                          $this->table, $this->owner,
                          $this->dbh->quoteString($userdata['nickname']),
                          $this->dbh->quoteString($userdata['firstname']),
-                         $this->dbh->quoteString($userdata['lastname']),
+                         $this->dbh->quoteString((!empty($userdata['lastname'])?$userdata['lastname']:'')),
                          $this->dbh->quoteString($userdata['email']),
-                         $this->dbh->quoteString($userdata['label']) );
+                         $this->dbh->quoteString((!empty($userdata['label'])?$userdata['label']:'')) );
 
          /* Do the insert */
          $r = $this->dbh->simpleQuery($query);
-         if ($r == DB_OK) {
-             return true;
+
+         /* Check for errors */
+         if (DB::isError($r)) {
+             return $this->set_error(sprintf(_("Database error: %s"),
+                                             DB::errorMessage($r)));
          }
 
-         /* Fail */
-         return $this->set_error(sprintf(_("Database error: %s"),
-                                         DB::errorMessage($r)));
+         return true;
     }
 
     /**
@@ -346,7 +408,7 @@ class abook_database extends addressbook_backend {
      */
     function remove($alias) {
         if (!$this->writeable) {
-            return $this->set_error(_("Addressbook is read-only"));
+            return $this->set_error(_("Address book is read-only"));
         }
 
         if (!$this->open()) {
@@ -367,13 +429,13 @@ class abook_database extends addressbook_backend {
 
         /* Delete entry */
         $r = $this->dbh->simpleQuery($query);
-        if ($r == DB_OK) {
-            return true;
-        }
 
-        /* Fail */
-        return $this->set_error(sprintf(_("Database error: %s"),
-                                         DB::errorMessage($r)));
+        /* Check for errors */
+        if (DB::isError($r)) {
+            return $this->set_error(sprintf(_("Database error: %s"),
+                                            DB::errorMessage($r)));
+        }
+        return true;
     }
 
     /**
@@ -384,7 +446,7 @@ class abook_database extends addressbook_backend {
      */
     function modify($alias, $userdata) {
         if (!$this->writeable) {
-            return $this->set_error(_("Addressbook is read-only"));
+            return $this->set_error(_("Address book is read-only"));
         }
 
         if (!$this->open()) {
@@ -394,8 +456,17 @@ class abook_database extends addressbook_backend {
          /* See if user exist */
         $ret = $this->lookup($alias);
         if (empty($ret)) {
-            return $this->set_error(sprintf(_("User %s does not exist"),
-                        '&qout;' . $alias . '&qout;'));
+            return $this->set_error(sprintf(_("User \"%s\" does not exist"), $alias));
+        }
+
+        /* make sure that new nickname is not used */
+        if (strtolower($alias) != strtolower($userdata['nickname'])) {
+            /* same check as in add() */
+            $ret = $this->lookup($userdata['nickname']);
+            if (!empty($ret)) {
+                $error = sprintf(_("User '%s' already exist."), $ret['nickname']);
+                return $this->set_error($error);
+            }
         }
 
         /* Create query */
@@ -405,23 +476,22 @@ class abook_database extends addressbook_backend {
                          $this->table,
                          $this->dbh->quoteString($userdata['nickname']),
                          $this->dbh->quoteString($userdata['firstname']),
-                         $this->dbh->quoteString($userdata['lastname']),
+                         $this->dbh->quoteString((!empty($userdata['lastname'])?$userdata['lastname']:'')),
                          $this->dbh->quoteString($userdata['email']),
-                         $this->dbh->quoteString($userdata['label']),
+                         $this->dbh->quoteString((!empty($userdata['label'])?$userdata['label']:'')),
                          $this->owner,
                          $this->dbh->quoteString($alias) );
 
         /* Do the insert */
         $r = $this->dbh->simpleQuery($query);
-        if ($r == DB_OK) {
-            return true;
-        }
 
-        /* Fail */
-        return $this->set_error(sprintf(_("Database error: %s"),
-                                        DB::errorMessage($r)));
+        /* Check for errors */
+        if (DB::isError($r)) {
+            return $this->set_error(sprintf(_("Database error: %s"),
+                                            DB::errorMessage($r)));
+        }
+        return true;
     }
 } /* End of class abook_database */
 
 // vim: et ts=4
-?>
\ No newline at end of file
