@@ -3,15 +3,15 @@
 /**
  * constants.php
  *
- * Copyright (c) 1999-2006 The SquirrelMail Project Team
- * Licensed under the GNU GPL. For full terms see the file COPYING.
- *
- * Loads constants used by the rest of the Squirrelmail source.
+ * Loads constants used by the rest of the SquirrelMail source.
  * This file is include by src/login.php, src/redirect.php and
  * src/load_prefs.php.
  *
- * @version $Id: constants.php,v 1.18.2.4 2006/02/03 22:27:47 jervfors Exp $
+ * @copyright &copy; 1999-2007 The SquirrelMail Project Team
+ * @license http://opensource.org/licenses/gpl-license.php GNU Public License
+ * @version $Id: constants.php 13185 2008-06-19 01:07:28Z pdontthink $
  * @package squirrelmail
+ * @since 1.2.0
  */
 
 /** Need to enable plugin functions for a hook */
@@ -53,6 +53,12 @@ define('SMPREF_JS_OFF', 0);
 define('SMPREF_JS_ON', 1);
 define('SMPREF_JS_AUTODETECT', 2);
 
+/* Define constants for address book functionalities. */
+define('SM_ABOOK_FIELD_NICKNAME', 0);
+define('SM_ABOOK_FIELD_FIRSTNAME', 1);
+define('SM_ABOOK_FIELD_LASTNAME', 2);
+define('SM_ABOOK_FIELD_EMAIL', 3);
+define('SM_ABOOK_FIELD_LABEL', 4);
+
 do_hook('loading_constants');
 
-?>
\ No newline at end of file
