@@ -3,16 +3,19 @@
 /**
  * options.php
  *
- * Copyright (c) 1999-2006 The SquirrelMail Project Team
- * Licensed under the GNU GPL. For full terms see the file COPYING.
- *
  * Displays the options page. Pulls from proper user preference files
  * and config.php. Displays preferences as selected and other options.
  *
- * @version $Id: options.php,v 1.121.2.13 2006/02/03 22:27:55 jervfors Exp $
+ * @copyright &copy; 1999-2007 The SquirrelMail Project Team
+ * @license http://opensource.org/licenses/gpl-license.php GNU Public License
+ * @version $Id: options.php 13241 2008-07-20 20:46:37Z pdontthink $
  * @package squirrelmail
+ * @subpackage prefs
  */
 
+/** This is the options page */
+define('PAGE_NAME', 'options');
+
 /**
  * Path for SquirrelMail required files.
  * @ignore
@@ -44,6 +47,19 @@ define('SMOPT_PAGE_HIGHLIGHT', 'highlight');
 define('SMOPT_PAGE_FOLDER', 'folder');
 define('SMOPT_PAGE_ORDER', 'order');
 
+/**
+  * Save submitted options and calculate the most 
+  * we need to refresh the page
+  *
+  * @param string $optpage      The name of the page being submitted
+  * @param array  $optpage_data An array of all the submitted options
+  *
+  * @return int The highest level of screen refresh needed per
+  *             the options that were changed.  This value will
+  *             correspond to the SMOPT_REFRESH_* constants found
+  *             in functions/options.php.
+  *
+  */
 function process_optionmode_submit($optpage, $optpage_data) {
     /* Initialize the maximum option refresh level. */
     $max_refresh = SMOPT_REFRESH_NONE;
@@ -264,6 +280,7 @@ if ($optmode == SMOPT_MODE_SUBMIT) {
 
 if ($optmode == SMOPT_MODE_SUBMIT) {
     $optpage = SMOPT_PAGE_MAIN;
+    $optpage_title = _("Options");
 }
 
 /***************************************************************/
@@ -470,4 +487,4 @@ if ($optpage == SMOPT_PAGE_MAIN) {
 </table>
 </td></tr>
 </table>
-</body></html>
\ No newline at end of file
+</body></html>
