@@ -3,14 +3,13 @@
 /**
  * view_text.php -- Displays the main frameset
  *
- * Copyright (c) 1999-2006 The SquirrelMail Project Team
- * Licensed under the GNU GPL. For full terms see the file COPYING.
- *
  * Who knows what this file does. However PUT IT HERE DID NOT PUT
  * A SINGLE FREAKING COMMENT IN! Whoever is responsible for this,
  * be very ashamed.
  *
- * @version $Id: view_text.php,v 1.17.2.8 2006/02/03 22:27:55 jervfors Exp $
+ * @copyright &copy; 1999-2007 The SquirrelMail Project Team
+ * @license http://opensource.org/licenses/gpl-license.php GNU Public License
+ * @version $Id: view_text.php 12581 2007-08-12 19:08:47Z kink $
  * @package squirrelmail
  */
 
@@ -32,7 +31,7 @@ sqgetGlobalVar('username',   $username,     SQ_SESSION);
 sqgetGlobalVar('onetimepad', $onetimepad,   SQ_SESSION);
 sqgetGlobalVar('delimiter',  $delimiter,    SQ_SESSION);
 sqgetGlobalVar('QUERY_STRING', $QUERY_STRING, SQ_SERVER);
-sqgetGlobalVar('messages', $messages);
+sqgetGlobalVar('messages', $messages,       SQ_SESSION);
 sqgetGlobalVar('passed_id', $passed_id, SQ_GET);
 
 if ( sqgetGlobalVar('mailbox', $temp, SQ_GET) ) {
@@ -64,6 +63,8 @@ $encoding = strtolower($header->encoding);
 $msg_url   = 'read_body.php?' . $QUERY_STRING;
 $msg_url   = set_url_var($msg_url, 'ent_id', 0);
 $dwnld_url = '../src/download.php?' . $QUERY_STRING . '&amp;absolute_dl=true';
+$unsafe_url = 'view_text.php?' . $QUERY_STRING;
+$unsafe_url = set_url_var($unsafe_url, 'view_unsafe_images', 1);
 
 $body = mime_fetch_body($imapConnection, $passed_id, $ent_id);
 $body = decodeBody($body, $encoding);
@@ -76,11 +77,13 @@ if (isset($languages[$squirrelmail_language]['XTRA_CODE']) &&
 }
 
 if ($type1 == 'html' || (isset($override_type1) &&  $override_type1 == 'html')) {
-    $body = MagicHTML( $body, $passed_id, $message, $mailbox);
+    $ishtml = TRUE;
     // html attachment with character set information
     if (! empty($charset))
         $body = charset_decode($charset,$body,false,true);
+    $body = magicHTML( $body, $passed_id, $message, $mailbox);
 } else {
+    $ishtml = FALSE;
     translateText($body, $wrap_at, $charset);
 }
 
@@ -94,6 +97,9 @@ echo _("Viewing a text attachment") . ' - ' .
 ?>
 </b></td><tr><tr><td><center>
 <?php
+if ( $ishtml ) {
+    echo '<a href="' . $unsafe_url . '">' . _("View Unsafe Images") . '</a> | ';
+}
 echo '<a href="' . $dwnld_url . '">' . _("Download this as a file") . '</a>';
 ?>
 </center><br />
@@ -103,4 +109,4 @@ echo '<a href="' . $dwnld_url . '">' . _("Download this as a file") . '</a>';
 <tr><td bgcolor="<?php echo $color[4]; ?>"><tt>
 <?php echo $body; ?>
 </tt></td></tr></table>
-</body></html>
\ No newline at end of file
+</body></html>
